//package ThreeD.Loader3DS;

import java.io.*;
import javax.vecmath.*;

// data is stored with LSB first
public class Chunk{
	
    public Chunk(){
	fileOffset = 0;
	id = 0;
	length = 0;
    }
	
    public Chunk(RandomAccessFile in){
	try{
	    fileOffset = in.getFilePointer();
	    id = (short)readUnsignedShort(in);	    
	    length = readInt(in);
	} catch(IOException e){
	    System.err.println("Chunk(RandomAccessFile):");
	    System.err.println(e); System.exit(1);
	}
    }

    public int skip(RandomAccessFile in, int offset) throws IOException{
	in.seek(fileOffset);
	return in.skipBytes(length + offset);
    }	
	
    public Vector3f readVector3f(RandomAccessFile in) throws IOException{
	return(new Vector3f(readFloat(in), readFloat(in), readFloat(in)));
    }	

    public Color3f readColor3f(RandomAccessFile in) throws IOException{	
	return(new Color3f(readFloat(in), readFloat(in), readFloat(in)));
    }
    public Color3b readColor3b(RandomAccessFile in) throws IOException{
	return(new Color3b((byte)in.readUnsignedByte(), 
			   (byte)in.readUnsignedByte(),
			   (byte)in.readUnsignedByte()));
    }

    public float readFloat(RandomAccessFile in) throws IOException{
	return Float.intBitsToFloat(readInt(in));
    }
			
    public int readInt(RandomAccessFile in) throws IOException{	
	int temp = in.readInt();
	return((temp << 24) & 0xFF000000) | ((temp << 8) & 0x00FF0000) |
	    ((temp >> 8) & 0x0000FF00) | ((temp >> 24) & 0x000000FF);
    }
		
    public int readUnsignedShort(RandomAccessFile in) throws IOException{	
	int temp = in.readUnsignedShort();
	return ((temp << 8) & 0xFF00) | ((temp >> 8) & 0x00FF);
    }
	
    public String readString(RandomAccessFile in) throws IOException{		
	StringBuffer buff = new StringBuffer("");		
	byte ch = (byte)in.readUnsignedByte();
	while(ch != (byte)0) {
	    buff.append((char)ch);
	    ch = (byte)in.readUnsignedByte();
	}
	return(buff.toString());
    }
	
    public long fileOffset(){return fileOffset;};
    public short id(){return id;};    
    public int length(){return length;};
	
    public final static short HEADER_SIZE = 6;
	
    private long fileOffset;    
    private short id;
    private int length;

}// end Chunk class




