//package ThreeD.Loader3DS;

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;

import javax.media.j3d.*;
import javax.vecmath.*;
	
import com.sun.j3d.loaders.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.image.TextureLoader;
import com.sun.j3d.utils.geometry.*;

import Chunk;
import Material3DS;

public class Loader3DS extends LoaderBase{
	
    static boolean DEBUG = false;
    BranchGroup rootBranchGroup;    
    RandomAccessFile in;
    SceneBase scene;
    Hashtable materials;
    Point3f[] vertices;   //vertices of current object    
    Material3DS currentMat; //data for the current material chunk 
    BranchGroup currentObject;  //
    String[] spaces = {"", "  ", "    ", "      ", "        "};

    ColoringAttributes color = 
	new ColoringAttributes(0.5f, .5f, .6f, 
			       ColoringAttributes.SHADE_GOURAUD);
    PolygonAttributes patt = 
	new PolygonAttributes(PolygonAttributes.POLYGON_FILL,
			      PolygonAttributes.CULL_BACK, 0.0f);

    /*
    RenderingAttributes rendAttribs =
	new RenderingAttributes
    */

    Color3f emissive = new Color3f(0.4f, 0.4f, 0.4f);
    Color3f ambient = new Color3f(0.6f, 0.5f, 0.5f);
    Color3f diffuse = new Color3f(0.6f, 0.6f, 0.6f);
    Color3f specular = new Color3f(.8f, .8f, .8f);

    
    public Loader3DS(){
	super();
	materials = new Hashtable();
	basePath = "";
    }
	
    public Loader3DS(int flags){
	super(flags);
	materials = new Hashtable();
	basePath = "";
    }    

    Component component; // only used if we have to load a texture from a file
    public Loader3DS(Component c){
	super();
	materials = new Hashtable();
	basePath = "";
	component = c;
    }
				
    ////////////////////// read Main chunk ////////////////////////
    
    protected void readMainChunk(Chunk mainChunk) throws IOException{	
	int version, count = 0;

	if(DEBUG) System.out.println("called readMainChunk");
		
	if(mainChunk.id() != MAIN_3DS)
	    throw new IncorrectFormatException("bad main chunk: " + 
					       Integer.toHexString(mainChunk.id()));
	
	// make sure the entire primary chunk is here
	if(mainChunk.length() > in.length()) 
	    throw new ParsingErrorException("file length and main chunk " +
					    "size don't match: " + 
					    mainChunk.length());
	else if(DEBUG) System.out.println("main length: " + mainChunk.length());
			
	Chunk chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	while(count < mainChunk.length()){		
	    switch(chunk.id()){
	    case VERSION_3DS:					
		version = chunk.readInt(in); count += 4;
		if(DEBUG) System.out.println("version: " + version);
		if(version < 3)  throw new IncorrectFormatException();
		break;
	    case EDITOR_3DS:					
		count += readEditChunk(chunk); 
		break;
	    case (short)KEYFRAME_3DS:
		if((getFlags() & LOAD_BEHAVIOR_NODES) == LOAD_BEHAVIOR_NODES){
		    count += readKeyChunk(chunk); 
		}else count += readKeyChunk(chunk); 
		break;
	    default:
		if(DEBUG)
		    System.out.println("unknown subchunk of MAIN_3DS: " + 
				       Integer.toHexString(chunk.id()));
		count += chunk.skip(in, -1);
		break;
	    }				    
	    if(count < mainChunk.length()){
		chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
	    }
	}
	if(DEBUG) System.out.println("finished reading Main chunk: " + count);
    }
	
    //////////////////////// read Editor Chunk ///////////////////////
	
    private int readEditChunk(Chunk editChunk) throws IOException{
	Chunk chunk = new Chunk(in);		
	int count = Chunk.HEADER_SIZE;

	while(count < editChunk.length()){
	    switch(chunk.id()){
	    case EDIT_OBJECT:
		if(DEBUG) System.out.println("\nObject Block");
		count += readObjectChunk(chunk);
		break;			
	    case (short)EDIT_MATERIAL:
		count += readMaterialChunk(chunk);
		break;
	    case EDIT_VIEW:   // this has more to do with the window
		count += chunk.skip(in, 0);
		//count += readViewChunk(chunk);		
		break;		
	    case EDIT_BACKGROUND_COLOR:		
		if((getFlags() & LOAD_BACKGROUND_NODES) == 
		   LOAD_BACKGROUND_NODES){
		    Chunk temp = chunk;
		    chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
		    if(chunk.id() == 0x0010){		    
			Background bg = new Background(chunk.readColor3f(in));
			bg.setApplicationBounds(new BoundingBox());
			scene.addBackgroundNode(bg);
		    }
		    chunk.skip(in, 0);		
		    temp.skip(in, 0);
		    chunk = temp;
		}else count += chunk.skip(in,0);
		break;
	    case EDIT_MESH_VERSION:
		//if(DEBUG) 
		    System.out.println("mesh version: " + 
					     chunk.readUnsignedShort(in));
		chunk.skip(in, 0);
		break;
	    default:
		if(DEBUG) System.out.println("unknown editor chunk: " + 
					      Integer.toHexString(chunk.id()));
		count += chunk.skip(in, 0);
		break;
	    }		    
	    if(count+1 < editChunk.length()){
		chunk = new Chunk(in); count+=Chunk.HEADER_SIZE;
	    }
	}			
	return editChunk.skip(in, 0);
    }	

    ////////////////////////  read Material chunk ////////////////////////

    private int readMaterialChunk(Chunk matChunk) throws IOException{
	int count = 0;//Chunk.HEADER_SIZE;
	if(DEBUG) System.out.println(spaces[1] + "reading Material Chunk: " + matChunk.length());
	currentMat = new Material3DS();
	Chunk chunk = new Chunk(in); 	count += Chunk.HEADER_SIZE;	
	while(count < matChunk.length()){	
	    switch(chunk.id()){
	    case (short)MAT_NAME:
		String name = chunk.readString(in);
		if(DEBUG) System.out.println(spaces[2] + "material name: " + name);
		currentMat.name = name;
		count += chunk.skip(in,0);  // this doesnt need to be here
		break;
	    case (short)MAT_TEXTMAP01:  // no data, only subchunks
		if(DEBUG) System.out.println("    texture map");
		count += readMaterialSubChunk(chunk);
		break;	    
	    case (short)MAT_REND_TYPE:
		//if(DEBUG) 
		    System.out.println(spaces[2] + "render type: " 
				   + chunk.readUnsignedShort(in));
		count += chunk.skip(in,0);  // this doesnt need to be here
		break;
	    case (short)MAT_AMBIENT_RGBB:
		currentMat.setAmbientColor(chunk.readColor3b(in));
		//i think there is a another color chunk in here
		count += chunk.skip(in, 0);
		break;
	    case (short)MAT_DIFFUSE_RGBB:
		currentMat.setDiffuseColor(chunk.readColor3b(in));
		//i think there is a another color chunk in here
		count += chunk.skip(in, 0);
		break;
	    case (short)MAT_SPECULAR_RGBB:
		currentMat.setSpecularColor(chunk.readColor3b(in));
		//i think there is a another color chunk in here
		count += chunk.skip(in, 0);
		break;
	    case (short)MAT_SHININESS:
		//System.out.println("shininess");
		Chunk temp = chunk;
		chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
		if(chunk.id() == 0x0030)
		    currentMat.setShininess(chunk.readUnsignedShort(in));
		if(chunk.id() == 0x0031)
		    currentMat.setShininess(chunk.readFloat(in));
		chunk.skip(in, 0);		
		temp.skip(in, 0);
		chunk = temp;
		break;
	    case (short)MAT_SHININESS_STRENGTH: 
	    case (short)MAT_REFLECTION_BLUR:
	    case (short)MAT_TRANSPARENCY_FALLOFF:
		count += chunk.skip(in, 0);
		break;
	    case (short)MAT_TRANSPARENCY:
		//percent sub chunk
		Chunk tempC = chunk;
		chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
		if(chunk.id() == 0x0030){
		    TransparencyAttributes t = 
		     new TransparencyAttributes(TransparencyAttributes.FASTEST,
						chunk.readUnsignedShort(in));
		    currentMat.setTransparencyAttributes(t);
		}
		chunk.skip(in, 0);		
		tempC.skip(in, 0);
		chunk = tempC;				
		count += chunk.skip(in, 0);
		break;
	    default:
		if(DEBUG) System.out.println(spaces[4] + "    unknown: " + 
					     Integer.toHexString(chunk.id()));
		count += chunk.skip(in, 0);
		break;
	    }	    
	    if(count < matChunk.length()){
		chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
		count -= 2;
	    }
	}
	
	materials.put(currentMat.name, currentMat);
	currentMat = null;
	return matChunk.skip(in, 0);
    }

    ////////////////////////////////////////////////////////////
       
    private int readMaterialSubChunk(Chunk matSubChunk) throws IOException{
	int count = 0;
	//	System.out.println(spaces[3] + "reading Mapping Chunk: ");
	Chunk chunk = new Chunk(in);
	while(count < matSubChunk.length()){
	    switch(chunk.id()){	    
	    case (short)MAT_FILENAME:
		currentMat.filename = chunk.readString(in);
		if(DEBUG) System.out.println(spaces[4] + "filename: " + 
					     currentMat.filename);
		count += chunk.skip(in,0);  // this doesnt need to be here
		break;
	    default:
		count += chunk.skip(in,0);
		//System.out.println(spaces[4] + "    unknown sub: " + 
		// 			   Integer.toHexString(chunk.id()));
		break;
	    }
	    if(count < matSubChunk.length()){
		chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	    }
	}
	
	return matSubChunk.skip(in, 0);
    }
    

    ////////////////////////////////////////////////////////////

    private int readViewChunk(Chunk viewChunk) throws IOException{
	int temp, count = 0;
	currentObject = new BranchGroup();

	Chunk chunk = new Chunk(in);
	while(count < viewChunk.length()){
	    switch(chunk.id()){
	    case VIEWPORT_SIZE:
		
		//if(DEBUG) 
		    System.out.println("view port size: x=" + 
				   chunk.readUnsignedShort(in) + ", x=" +
				   chunk.readUnsignedShort(in) + ", y=" +
				   chunk.readUnsignedShort(in) + ", w=" +
				   chunk.readUnsignedShort(in) + ", h=");
		count += chunk.skip(in, 0);
		break;
	    case VIEWPORT_DATA:
		chunk.readUnsignedShort(in); chunk.readUnsignedShort(in);
		chunk.readUnsignedShort(in); chunk.readUnsignedShort(in);
		chunk.readUnsignedShort(in); chunk.readUnsignedShort(in);
		chunk.readUnsignedShort(in); chunk.readFloat(in);
		//if(DEBUG) 
		    System.out.println("world coords: " + 
					     chunk.readFloat(in) + 
					     "," + chunk.readFloat(in) + ", " +
					     chunk.readFloat(in));
		chunk.readFloat(in); chunk.readFloat(in);
		//if(DEBUG) 
		    System.out.println("  name: " + chunk.readString(in));
		count += chunk.skip(in, 0);
		break;
	    case VIEWPORT_DATA_3:
		count += chunk.skip(in, 0);
		break;
	    }
	    if(count < viewChunk.length()){
		chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	    }
	}
	return viewChunk.skip(in, 0);
    }

    ///////////////// read Object Chunk //////////////////////
    
    // an object chunk contains a string which is the name of the object
    // immediately followd by one or more subchunks to describe the object
    private int readObjectChunk(Chunk objChunk) throws IOException{
	String objName = objChunk.readString(in);
	Chunk chunk = new Chunk(in);				
	int temp, count = objName.length() + 1;
	currentObject = new BranchGroup();
	
	//if(DEBUG_ON)  
	if(DEBUG) System.out.println("  objName: " + objName);
	
	while(count < objChunk.length()){
	    switch(chunk.id()){
	    case OBJ_TRIMESH:
		count += readTriMeshChunk(chunk);
		//System.out.println("skipping TriMesh object");
		break;

	    case OBJ_LIGHT:
		if((LOAD_LIGHT_NODES & getFlags()) == LOAD_LIGHT_NODES) 
		    count += readLightChunk(chunk);
		else count += chunk.skip(in,0);
		break;
	    case OBJ_CAMERA:   // read in the camera 
		if((LOAD_VIEW_GROUPS & getFlags()) == LOAD_VIEW_GROUPS){
				//Vector3f camera = chunk.readVector3f(in);
		    Point3f camera = new Point3f(chunk.readVector3f(in));
		    if(DEBUG) System.out.println("  camera: " + camera);
		    Vector3f target = chunk.readVector3f(in);
//  		System.out.println("  target: " + target);
//  		float angle = camera.angle(target);
//  		System.out.println("  angle: " + angle);
				//float bank_angle = chunk.readFloat(in);
				//float focus = chunk.readFloat(in);; 	
		    count += chunk.skip(in, 0) + 6;
		    PhysicalBody body = new PhysicalBody(new Point3d(camera), 
							 new Point3d(camera));
		    View view = new View();
		    view.setPhysicalBody(body);
		    ViewPlatform platform = new ViewPlatform();
		    view.attachViewPlatform(platform);		
		    TransformGroup tg = new TransformGroup();
		    tg.addChild(platform);
		    scene.addViewGroup(tg);
		} else count += chunk.skip(in, 0);
		break;
	    case OBJ_B_HIDDEN: //object is hidden
		if(DEBUG) System.out.println("  object hidden: ");
		//currentMat.setHidden(true);
		//chunk.skip(in, 0);
		break;
		// boolean value whose existence will indicate a true value
		/*case 0x4010:*/ case 0x4011: case 0x4012: case 0x4013:
	    case 0x4014: case 0x4015: case 0x4016: case 0x4017: 	
		if(DEBUG) System.out.println("bool: " + 
					     Integer.toHexString(chunk.id()));
		break;
	    default:
		if(DEBUG)  System.out.println("unknown object chunk: " + 
					      Integer.toHexString(chunk.id()));
		temp = chunk.skip(in, 0);
		count += temp + 6;//skipChunk(in, 7);// + 6;
		break;
	    }
	    //System.out.println("obj count: " + count);
	    if(count+1 < objChunk.length()){			
		chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	    }
	}				
	
	scene.addNamedObject(objName, currentObject);
	rootBranchGroup.addChild(currentObject);

	return objChunk.skip(in, 0);	
    }

    ////////////////////////////////////////////////////////////

    private int readLightChunk(Chunk lightChunk) throws IOException{
	int count = Chunk.HEADER_SIZE;
	PointLight light = new PointLight();	
	light.setEnable(true);
	light.setColor(new Color3f(1f,1f,1f));

	if(DEBUG) System.out.println("  entered readLightChunk()");

	Vector3f pos = lightChunk.readVector3f(in);
	light.setPosition(pos.x, pos.y, pos.z);
	if(DEBUG) System.out.println("  pos: " + pos);
	
	Chunk chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	while(count <lightChunk.length()){
	    switch(chunk.id()){
	    case LIGHT_SPOT:
		if(DEBUG) System.out.println("    spot light");
		Vector3f target = chunk.readVector3f(in);
		float hotspot = chunk.readFloat(in);
		float fallOff = chunk.readFloat(in);		
		light = new SpotLight(true, new Color3f(1.0f, 1.0f, 1.0f), 
				      new Point3f(pos), new Point3f(1,0,0), 
				      target, (float)Math.PI, 0.0f) ;
		light.setInfluencingBounds(new BoundingSphere());
		count += chunk.skip(in,0);
		break;
	    case LIGHT_OFF:
		if(DEBUG) System.out.println("    light off");
		light.setEnable(false);
		count += chunk.skip(in,0);
	    default:
		count += chunk.skip(in,0);
		break;
	    }
	    if(count < lightChunk.length()){
		chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
	    }		
	}		   	
	scene.addLightNode(light);
	return lightChunk.skip(in, 0);
    }
	
    /////////////////////////// read TriMesh chunk //////////////////////

    private int readTriMeshChunk(Chunk triChunk) throws IOException{
	int nflags;						
	int count = Chunk.HEADER_SIZE;
		
	if(DEBUG) System.out.println("    readTriMeshChunk() called: " + 
				     triChunk.length());
		
	Chunk chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
	while(count < triChunk.length()){
	    switch(chunk.id()){				
	    case TRI_VERTEX_LIST:
		if(DEBUG) System.out.println("  reading vertices...");
		count += readVertexListChunk(chunk);
		break;
	    case TRI_FACES_ARRAY:
		if(DEBUG) System.out.println("  reading faces...");
		count += readFacesArrayChunk(chunk);		
		break;
	    case TRI_VERTEX_MAP_LIST:		
		count += chunk.skip(in, 0);	break;
	    case TRI_LOCAL_COORDS:		
		count += chunk.skip(in, 0);	break;
	    case TRI_VISIBLE:		
		count += chunk.skip(in, 0);	break;
	    case 0x4181: case0x4182:
		if(DEBUG) System.out.println("skipping *** WEIRD *****");
		in.skipBytes(-4);
		count += chunk.skip(in, 0);	break;
	    default:
		count += chunk.skip(in, 0);	break;
	    }	    
	    if(count < triChunk.length()){
		chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	    }
	}

	//System.out.println("returning from readTriMeshChunk");
	//return count;
	return triChunk.skip(in, 0);
    }

    ///////////////// read Vertex List Chunk //////////////////////

    private int readVertexListChunk(Chunk vertChunk) throws IOException{
	int npoints = vertChunk.readUnsignedShort(in);
	if(DEBUG) System.out.println("    readVertexListChunk() called: " + 
					npoints);
	vertices = new Point3f[npoints];
	for(short i=0; i<npoints; i++){
	    vertices[i] = new Point3f(vertChunk.readVector3f(in));
   	    float temp = vertices[i].z;
   	    vertices[i].z = -vertices[i].y;
   	    vertices[i].y = temp;
	}
	return vertChunk.skip(in,0);
    }
	
    ///////////////// read Faces Array Chunk //////////////////////

    private int readFacesArrayChunk(Chunk facesChunk) throws IOException{
	int count = Chunk.HEADER_SIZE;
	int nFaces = facesChunk.readUnsignedShort(in); count += 2;
	if(DEBUG) 
	    System.out.println("    readFacesArrayChunk() called: " + nFaces);

	int[] strips = new int[1];  strips[0] = nFaces*3;
	//String[] matName = null;
	LinkedList matNames = new LinkedList();
	IndexedTriangleStripArray tri = 
	    new IndexedTriangleStripArray(vertices.length,
					IndexedTriangleStripArray.COORDINATES |
					IndexedTriangleStripArray.NORMALS,
					nFaces*3, strips);
	tri.setCoordinates(0,vertices);
	tri.setNormals(0, generateNormals(vertices));

	//get the index numbers for the faces
	for(int i=0; i<nFaces; i++){
	    int index[] = new int[3];
	    index[0] = facesChunk.readUnsignedShort(in); count += 2;
	    index[1] = facesChunk.readUnsignedShort(in); count += 2;
	    index[2] = facesChunk.readUnsignedShort(in); count += 2;

	    count += 2;
	    facesChunk.readUnsignedShort(in); // flag for this face 

	    tri.setCoordinateIndices(i*3, index);
	    tri.setNormalIndices(i*3, index);
	}

	String matName = null;
	//read sub chunks of face chunk (0x4130, 0x4150)	
	Chunk chunk = new Chunk(in); count += Chunk.HEADER_SIZE;
	while(count < facesChunk.length()){
	    switch(chunk.id()){
	    case FACES_MAT_LIST:  //0x4130
		matName = chunk.readString(in); 
		int nEntries = chunk.readUnsignedShort(in);
		String[] nameList = new String[2];
		nameList[0] = matName; 
		nameList[1] = Integer.toString(nEntries);
		matNames.add(nameList);		
		if(DEBUG) System.out.println("    material name: " + matName +
				   ", " + nEntries);
// 		for(int i=0; i<nEntries; i++) 
// 		    System.out.println("    face #: " + 
// 				       chunk.readUnsignedShort(in));
		count += chunk.skip(in , 0);
		break;		
	    case FACES_SMOOTH:   //0x4150
		if(DEBUG) System.out.println("*** got faces SMOOTH");
		count += chunk.skip(in , 0);
		break;
	    }
	    if(count < facesChunk.length()){
		chunk = new Chunk(in);  count += Chunk.HEADER_SIZE;
	    }
	}	

	// we now have enough info to create the face
	constructFace(tri, matName);  
	return facesChunk.skip(in, 0);
    }

    ////////////////////////////////////////////////////////////

    // create a face as a child of currentObject from the TriangleStripArray 
    // and properties from the Material3DS with name <matName>
    // ? can there be one than one material per face ?    
    private void constructFace(GeometryArray tri, String matName){
//     private void constructFace(GeometryArray tri, LinkedList matNames){
	Material3DS mat = new Material3DS();
	Appearance appear = new Appearance();

	if(DEBUG) System.out.println("  constructing Face: ");
       
	// if a material exists for this face
	if(matName != null){ 
	    mat = (Material3DS)materials.get(matName);
	    // if material has a texture
	    if(mat.filename != null && component != null){ 
		if(DEBUG) System.out.println("      loading " + basePath + 
				   mat.filename.toLowerCase());
		//Image img = component.getToolkit().getImage(basePath + mat.filename.toLowerCase());
		if(DEBUG) System.out.println("   got image");
//  		TextureLoader texLoad = new TextureLoader(img, component);
// 		System.out.println("   created TextureLoader");
// 		appear.setTexture(texLoad.getTexture());
//  		appear.setTexture(new TextureLoader(img, component).getTexture());
		if(DEBUG) System.out.println("   got texture");		
// 		appear.setTexCoordGeneration(new TexCoordGeneration());
// 		TextureAttributes t = new TextureAttributes();
// 		t.setTextureMode(TextureAttributes.);
// 		appear.setTextureAttributes(t);
		//appear.setTextureAttributes(new TextureAttributes());
	    } //end if(texture file)
	    else{
// 		appear.setTexture(new Texture2D());
// 		appear.setTexCoordGeneration(new TexCoordGeneration());    
// 		appear.setTextureAttributes(new TextureAttributes());
	    }
 	    appear.setMaterial(mat.toMaterial());
// 	    appear.setMaterial(new Material(ambient, emissive, diffuse, 
// 					    specular, 64));
	    appear.setTransparencyAttributes(mat.getTransparencyAttributes());

	    RenderingAttributes rendAttribs = 
		new RenderingAttributes(true, false, (float)0.0, 
					RenderingAttributes.ALWAYS);	    
					//RenderingAttributes.GREATER);	    
	    appear.setRenderingAttributes(rendAttribs);	    
	    
	}// end if(matName != null)
	else{ // no material defined
// 	    appear.setMaterial(new Material(ambient, emissive, diffuse, 
// 					    specular, 64));
		
	}

	appear.setColoringAttributes(color);
	appear.setPolygonAttributes(patt);
	if(!mat.getHidden()) // is object hidden(not yet working)
	    currentObject.addChild(new Shape3D(tri, appear));
    }



    ////////////////// read KeyFramer chunk ////////////////////

    private int readKeyChunk(Chunk keyChunk) throws IOException{
	int count;
	//System.out.println("readKeyChunk() called: " + keyChunk.length());
	count = keyChunk.skip(in, 0);
	return count;		
    }
		
    ////////////////////////////////////////////////////////////i

    public Scene load(String fileName) throws FileNotFoundException, 
	IncorrectFormatException, ParsingErrorException{	
	rootBranchGroup = new BranchGroup();
	scene = new SceneBase();	
	in = new RandomAccessFile(basePath + fileName, "r");

	scene.addDescription(fileName); 
	if(basePath.equals(""))  basePath = getBaseName(fileName);	
	if(DEBUG) System.out.println("basePath: " + basePath);

	try{				
	    Chunk chunk = new Chunk(in);
	    readMainChunk(chunk);				
	} catch(IOException e){
	    System.err.println("Loader3DS.Scene(String): ");
	    System.err.println(e);				
	    System.exit(1);
	} 

	scene.setSceneGroup(rootBranchGroup);		
	return scene;
    }

    // load the scene from <url>
    public Scene load(URL url) throws FileNotFoundException, 
	IncorrectFormatException, ParsingErrorException{
	rootBranchGroup = new BranchGroup();
	scene = new SceneBase();

	if(DEBUG) System.out.println("get file from URL");
	//in = new RandomAccessFile(url.getFile(), "r");	   
	in = new RandomAccessFile("/home/adams/code/java/models/v-38/v-38.3ds",
				  "r");
	if(DEBUG) System.out.println("created RandomAccess file");
	try{				
	    Chunk chunk = new Chunk(in);
	    readMainChunk(chunk);				
	} catch(IOException e){
	    System.err.println("Loader3DS.Scene(String): ");
	    System.err.println(e);				
	    System.exit(1);
	} 

	scene.setSceneGroup(rootBranchGroup);		
	return scene;	
    }

    ////////////////////////////////////////////////////////////

    public Scene load(Reader reader) throws FileNotFoundException, 
	IncorrectFormatException, 
	ParsingErrorException{
	if(DEBUG) System.out.println("Loader3DS.load(Reader): " + 
			   "loading from a Reader object is not supported");
	System.exit(1);
	return null;
    }

    ////////////////////////////////////////////////////////////

    //return an array of normals for the Tuples passed
    private Vector3f[] generateNormals(Tuple3f[] p){
	Vector3f[] v = new Vector3f[p.length];
	for(int i=0; i<p.length; i++){
	    v[i] = new Vector3f(p[i]);
	    v[i].normalize();
	}
	return v;
    }
	    
    ////////////////////////////////////////////////////////////

    // utility function to extract the base path ; used by load(String) so we  
    // know where to find the files referenced by the model file
    private String getBaseName(String fileName){	
	try{  
	    return fileName.
		substring(0, fileName.lastIndexOf(File.separatorChar) + 1);
	}catch(StringIndexOutOfBoundsException e){return "";}
    }    

    ////////////////////////////////////////////////////////////

    public void dispose() throws IOException{
	;		// does nothing
    }    

    ////////////////////////////////////////////////////////////
    
    // Id Chunk
    final static int MAIN_3DS = 0x4D4D;
    final static int VERSION_3DS = 0x0002;	
    

    // Main Chunks
    final static int EDITOR_3DS = 0x3D3D;  
    // this is the start of the editor config
    final static int KEYFRAME_3DS = 0xB000;  
    // this is the start of the keyframer config

    // sub defines of EDIT3DS
    final static int EDIT_MATERIAL = 0xAFFF;
    final static int EDIT_CONFIG1  = 0x0100;
    final static int EDIT_CONFIG2  = 0x3E3D;

    final static int EDIT_MESH_VERSION = 0x3D3E;

    final static int EDIT_VIEW    = 0x7001;
    final static int VIEWPORT_DATA  = 0x7011;
    final static int VIEWPORT_DATA_3  = 0x7012;    
    final static int VIEWPORT_SIZE  = 0x7020;
    
    final static int EDIT_BACKGROUND_BITMAP   = 0x1100;
    final static int EDIT_BACKGROUND_COLOR   = 0x1200;    
    final static int EDIT_AMBIENT  = 0x2100;
    final static int EDIT_OBJECT   = 0x4000;    

    //sub defines of EDIT_MATERIAL
    final static int MAT_NAME    = 0xA000;
    final static int MAT_FILENAME    = 0xA300;
    final static int MAT_TEXTMAP01    = 0xA200;
    final static int MAT_REND_TYPE    = 0xA100;
    final static int MAT_AMBIENT_RGBB = 0xA010;
    final static int MAT_DIFFUSE_RGBB = 0xA020;
    final static int MAT_SPECULAR_RGBB = 0xA030;
    final static int MAT_SHININESS     = 0xA040;
    final static int MAT_SHININESS_STRENGTH     = 0xA041;
    final static int MAT_TRANSPARENCY     = 0xA050;
    final static int MAT_TRANSPARENCY_FALLOFF     = 0xA052;
    final static int MAT_REFLECTION_BLUR     = 0xA053;
    
    // sub defines of EDIT_OBJECT
    final static int OBJ_TRIMESH   = 0x4100;
    final static int OBJ_LIGHT     = 0x4600;
    final static int OBJ_CAMERA    = 0x4700;

    final static int OBJ_B_HIDDEN  = 0x4010;
    final static int OBJ_B_NOCAST  = 0x4012; 

    // sub defines of OBJ_CAMERA
    final static int CAM_UNKNWN01  = 0x4710; 
    final static int CAM_UNKNWN02  = 0x4720; 

    // sub defines of OBJ_LIGHT
    final static int LIGHT_OFF       = 0x4620;
    final static int LIGHT_SPOT      = 0x4610;
    //final static int LIGHT_UNKNWN01  = 0x465A;

    // defines of OBJ_TRIMESH
    final static int TRI_VERTEX_LIST	= 0x4110;
    final static int TRI_FACES_ARRAY	= 0x4120;
    final static int FACES_MAT_LIST		= 0x4130;
    final static int FACES_SMOOTH    		= 0x4150;
    final static int TRI_VERTEX_MAP_LIST	= 0x4140;    
    final static int TRI_LOCAL_COORDS     	= 0x4160;
    final static int TRI_VISIBLE   		= 0x4165;

    // sub defs of KEYFRAME_3DS
    final static int KEYF_UNKNWN01 = 0xB009;
    final static int KEYF_UNKNWN02 = 0xB00A;
    final static int KEYF_FRAMES   = 0xB008;
    final static int KEYF_OBJDES   = 0xB002;

    final static int KEYF_OBJHIERARCH  = 0xB010;
    final static int KEYF_OBJDUMMYNAME = 0xB011;
    final static int KEYF_OBJUNKNWN01  = 0xB013;
    final static int KEYF_OBJUNKNWN02  = 0xB014;
    final static int KEYF_OBJUNKNWN03  = 0xB015; 
    final static int KEYF_OBJPIVOT     = 0xB020;
    final static int KEYF_OBJUNKNWN04  = 0xB021; 
    final static int KEYF_OBJUNKNWN05  = 0xB022;  

    // the different color chunk types
    final static int COL_RGB  = 0x0010;
    final static int COL_TRU  = 0x0011;
    final static int COL_RGB_FULL  = 0x0012;   // gamma corrected value
    final static int COL_TRU_FULL  = 0x0013;   // gamma corrected value

    // percent chunks
    final static int PERCENT_F = 0x0031;
    final static int PERCENT_I = 0x0030;

    // viewport chunks
    final static int TOP           = 0x0001;
    final static int BOTTOM        = 0x0002;
    final static int LEFT          = 0x0003;
    final static int RIGHT         = 0x0004;
    final static int FRONT         = 0x0005;
    final static int BACK          = 0x0006;
    final static int USER          = 0x0007;
    final static int CAMERA        = 0x0008;//0xFFFF is the code read from file
    final static int LIGHT         = 0x0009;
    final static int DISABLED      = 0x0010;
    final static int BOGUS         = 0x0011;
	
} //end Loader3DS
