/**
 * Material3DS.java
 *
 *
 * Created: Tue Jun 01 21:46:39 1999
 *
 * @author Brett Adams
 * @version
 */

import javax.vecmath.*;
import javax.media.j3d.*;

// material class for a 3D-Studio object, contains the colors as well as
// the name and filename of the texture
public class Material3DS{
    public Material3DS() 
    {
	// set defaults for this materials Appearance
	coloring = new ColoringAttributes();
	line = new LineAttributes();
	point = new PointAttributes();
	polygon = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,
					PolygonAttributes.CULL_BACK, 0.0f);	
	rendering = new RenderingAttributes(true, false, (float)0.0, 
					    RenderingAttributes.ALWAYS);	
	transparency = new TransparencyAttributes();
	textureAttrib = new TextureAttributes();
	
	
	ambientColor = new Color3b((byte)51, (byte)51, (byte)51);
	emmissiveColor = new Color3b((byte)127, (byte)127, (byte)127);
	//emmissiveColor = new Color3b((byte)0, (byte)0, (byte)0);
	diffuseColor = new Color3b((byte)255, (byte)255, (byte)255);
	specularColor = new Color3b((byte)255, (byte)255, (byte)255);
	shininess = 64;
	material = new Material(); 
    }
    
    public Material3DS(Color3b ambientColor, Color3b emmissiveColor,
			Color3b diffuseColor, Color3b specularColor,
			float shininess)
    {
	this.ambientColor = ambientColor;
	this.emmissiveColor = emmissiveColor;
	this.diffuseColor = diffuseColor;
	this.specularColor = specularColor;
	this.shininess = shininess;
    }
    
    public void setAmbientColor(Color3b color){ambientColor = color;}
    public void setEmmissiveColor(Color3b color){emmissiveColor = color;}
    public void setDiffuseColor(Color3b color){diffuseColor = color;}
    public void setSpecularColor(Color3b color){specularColor = color;}    
    public void setShininess(float shine){shininess = shine;}    
    public void setTransparencyAttributes(TransparencyAttributes t){
	transparency = t;
    }
    public void setHidden(boolean b){hidden = b;}
    public boolean getHidden(){return hidden;}
   
    public TransparencyAttributes getTransparencyAttributes(){return transparency;} 

    // converts the colors from RGB to true color and returns a material
    // object with our colors
    public Material toMaterial(){	
	Material m = new Material(toColor3f(ambientColor), 
				  toColor3f(emmissiveColor),
				  toColor3f(diffuseColor), 
				  toColor3f(specularColor), 
				  shininess);
	//m.setLightingEnable(false);
	return m;    
	//return new Material();
    }

    // this needs to be tested
    Color3b toColor3b(Color3f color){
	Color3b c = new Color3b();
	c.x = (byte)(color.x * 255);
	c.y = (byte)(color.y * 255);
	c.z = (byte)(color.z * 255);
	return c;
    }

    // this needs to be tested
    Color3f toColor3f(Color3b color){
	return new Color3f((float)color.x/255.0f, 
			   (float)color.x/255.0f, 
			   (float)color.x/255.0f);
    }

    public String name;
    public String filename;
    
    private Color3b ambientColor = null;
    private Color3b diffuseColor = null;
    private Color3b specularColor = null;
    private Color3b emmissiveColor = null;
    private float shininess;

    // attributes for this materials Appearance
    private ColoringAttributes coloring = null;
    private LineAttributes line = null;
    private PointAttributes point = null;
    private PolygonAttributes polygon = null;
    private RenderingAttributes rendering = null;
    private TransparencyAttributes transparency = null;
    private TextureAttributes textureAttrib = null;

    private Texture texture = null;

    private boolean hidden = false;
    private Material material;    
    
} // Material3DS




