/**
 * View3DS.java
 *
 *
 * Created: Sat Aug  7 10:37:57 1999
 *
 * @author brett adams
 * @version
 */


import java.io.*;
import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.util.*;

import javax.swing.event.*;
import javax.swing.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import com.sun.j3d.loaders.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;
import com.sun.j3d.utils.behaviors.keyboard.*;
import com.sun.j3d.utils.behaviors.mouse.*;

import Loader3DS;

public class View3DS extends JFrame implements ActionListener{
    
    public View3DS() {
	super("Java3D 3DS Viewer");
	setSize(600, 500);
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {System.exit(0);}
		public void windowOpened(WindowEvent e) {}});
	createUniverse();
	initGUI();
    }
    
    public static void main(String[] args) {	
	View3DS f = new View3DS();	
	f.setVisible(true);
    }
    
    BranchGroup model = null;
    SimpleUniverse u;
    Scene scene;

    JCheckBox bgBox;
    JCheckBox behaveBox;
    JCheckBox fogBox;
    JCheckBox liteBox;
    JCheckBox viewBox;
    JFileChooser fc;

    ////////////////////////////////////////////////////////////

    private void initGUI(){			   	
	JPanel rightP = new JPanel();

	rightP.setLayout(new BoxLayout(rightP, BoxLayout.Y_AXIS));
	bgBox = new JCheckBox("Load Backrounds", true);
	rightP.add(bgBox);
	behaveBox = new JCheckBox("Load Behaviors", true);
	rightP.add(behaveBox);
	fogBox  = new JCheckBox("Load Fog", true);
	rightP.add(fogBox);
	liteBox  = new JCheckBox("Load Lights", true);
	rightP.add(liteBox);
	//  	   box  = new JCheckBox("Load Sounds");
	viewBox  = new JCheckBox("Load Views", true);
	rightP.add(viewBox);
	//  	butt = new JButton("Camera");
	//    	butt.addActionListener(this);
	//  	rightP.add(butt);
	JButton b = new JButton("Open");
	rightP.add(b);
	b.addActionListener(this);

	getContentPane().add(rightP, "East");

    }

    ////////////////////////////////////////////////////////////

    private int getCheckedLoadFlags(){
	int flags = 0;
	if(bgBox.isSelected()) flags |= Loader3DS.LOAD_BACKGROUND_NODES;
	if(behaveBox.isSelected()) flags |= Loader3DS.LOAD_BEHAVIOR_NODES;
	if(fogBox.isSelected()) flags |= Loader3DS.LOAD_FOG_NODES;
	if(liteBox.isSelected()) flags |= Loader3DS.LOAD_LIGHT_NODES;
	if(viewBox.isSelected()) flags |= Loader3DS.LOAD_VIEW_GROUPS;
	return flags;
    }

    ////////////////////////////////////////////////////////////

    private void loadModel(String filename){	
	System.out.println("called loadModel()");
	//BranchGroup model = null;
	int flags = getCheckedLoadFlags();
	
	Loader3DS loader = new Loader3DS(this);
	loader.setFlags(flags);
	try{  
	    scene = loader.load(filename);	    
	} catch(Exception e){
	    System.err.println("Viewer.loadModel(): " + 
			       "Exception loading file: " + filename );
	    System.err.println(e);	System.exit(1);
	}
		
	System.out.println("file loaded");

	if(model != null)  model.detach();
	model = createBranchGraph();
	//model.setCapability(Group.ALLOW_BOUNDS_READ);
	model.setCapability(BranchGroup.ALLOW_DETACH);
	//model.setCapability(Group.ALLOW_CHILDREN_READ);
	model.compile();	
	u.addBranchGraph(model);	
	setView();       
	u.getViewer().getView().setBackClipDistance(10.0);
// 	u.getViewingPlatform().setNominalViewingTransform();
	System.out.println("model loaded...");
    }

    ////////////////////////////////////////////////////////////

    //create colored lines for the x,y,z axesx
    private BranchGroup createAxes(){
		// create axis
	LineArray axes = new LineArray(6, LineArray.COORDINATES | 
				       LineArray.COLOR_3);
	axes.setCoordinate(0, new Point3f(0f,0f,0f));
	axes.setCoordinate(1, new Point3f(100f,0f,0f));
 	axes.setColor(0, new Color3f(1f,0f,0f));
 	axes.setColor(1, new Color3f(1f,0f,0f));
	axes.setCoordinate(2, new Point3f(0f,0f,0f));
	axes.setCoordinate(3, new Point3f(0f,100f,0f));
 	axes.setColor(2, new Color3f(0f,1f,0f));
 	axes.setColor(3, new Color3f(0f,1f,0f));
	axes.setCoordinate(4, new Point3f(0f,0f,0f));
	axes.setCoordinate(5, new Point3f(0f,0f,100f));
 	axes.setColor(4, new Color3f(0f,0f,1f));
 	axes.setColor(5, new Color3f(0f,0f,1f));
	BranchGroup objRoot = new  BranchGroup();
	objRoot.addChild(new Shape3D(axes));
	return objRoot;
    }

    ////////////////////////////////////////////////////////////

    private void createUniverse(){
	System.out.println(" entered createUniverse()");
	Canvas3D canvas = new Canvas3D(null);	
	getContentPane().add("Center", canvas);
	u = new SimpleUniverse(canvas);	
	System.out.println("leaving createUniverse");	
    }

    ////////////////////////////////////////////////////////////

    private BranchGroup createBranchGraph(){		
	System.out.println("called createBranchGraph()");

	BranchGroup objRoot = new BranchGroup();	
	//now add the scene graph defined in the file
	if(scene.getSceneGroup() != null) {
	    TransformGroup objRotate = new TransformGroup();
	    objRotate.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	    objRotate.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

	    objRoot.addChild(objRotate);
	    objRotate.addChild(scene.getSceneGroup()); // root of model
	    objRotate.addChild(createAxes()); // create the axes for the scene

	    MouseRotate myMouseRotate = new MouseRotate(); 
	    myMouseRotate.setTransformGroup(objRotate);
	    myMouseRotate.setSchedulingBounds(new BoundingSphere()); 
	    objRoot.addChild(myMouseRotate);

	    TransformGroup vpTrans = 
		u.getViewingPlatform().getViewPlatformTransform();
	    KeyNavigatorBehavior keyNav = new KeyNavigatorBehavior(vpTrans);
	    keyNav.setSchedulingBounds(new BoundingSphere(new Point3d(),50));
	    // keyNav.setSchedulingBounds(new BoundingSphere());
	    objRoot.addChild(keyNav);

	    //set up light nodes if there are some
	    if(scene.getLightNodes() != null){		
		Light[] lights = scene.getLightNodes();
		System.out.println("adding " +lights.length + " lights nodes");
 		for(int i=0; i<lights.length; i++)
 		    objRoot.addChild(lights[i]);
	    }

	    //set up any background nodes
	    if(scene.getBackgroundNodes() != null){
		BranchGroup branch = new BranchGroup();		
		Background[] bg = scene.getBackgroundNodes();
		System.out.println("adding " + bg.length +" background nodes");
		Color3f c = new Color3f();
 		for(int i=0; i<bg.length; i++){
 		    branch.addChild(bg[i]);
		    bg[i].getColor(c);
		    System.out.println("color: " + c);
// 		    objRoot.addChild(bg[i]);
		}
		System.out.println("u.addBranchGraph()");
 		u.addBranchGraph(branch);
		System.out.println("u.addBranchGraph() done");
		//branch.addChild(new Background(new Color3f(1f,0f,0f)));
		//objRoot.addChild(new Background(new Color3f(1f,0f,0f)));
		//u.addBranchGraph(branch);
	    }

	    return objRoot;
	}else{
	    System.out.println("Viewer.createBranchGraph: "+
			       "could not create branch graph");
	    System.exit(1);
	}
	return null;
    }

    ////////////////////////////////////////////////////////////

    private void nextCamera(){
	System.out.println("next Camera called()");
	TransformGroup[] tg = scene.getViewGroups();
	System.out.println("num cameras: " + tg.length);
	return;
    }

    ////////////////////////////////////////////////////////////

    public void actionPerformed(ActionEvent event){
	System.out.println("action performed: ");
	System.out.println("action command: " + event.getActionCommand());
	System.out.println("action paramString: " + event.paramString());
	if(event.getActionCommand().equals("Open")){
    	    if(fc==null) fc = new JFileChooser();
    	    if(fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)		
		loadModel(fc.getSelectedFile().toString());
	}
	/*
	if(event.getActionCommand().equals("Camera"))
	    nextCamera();
	*/
    }

    ////////////////////////////////////////////////////////////

    private void setView(){

	//change the back clip distance
	View theView = u.getViewer().getView();
	theView.setBackClipDistance(50000f);
	
	// *********** temporary *******************
	// set a nice view, should be read in from the file
	// viewport data should eventually be read from the .3DS file
	ViewingPlatform view = u.getViewingPlatform();
	TransformGroup viewTransform = view.getViewPlatformTransform();	
	Transform3D translate = new Transform3D();
	Transform3D rotate = new Transform3D();	
	viewTransform.getTransform(translate);
	viewTransform.getTransform(rotate);
	rotate.rotY(Math.PI/8.0d);		
	Vector3f vector = new Vector3f();
	translate.get(vector);
	vector.z = vector.z + 30.0f;
	translate.setTranslation(vector);
	rotate.mul(translate); 
	viewTransform.setTransform(rotate);
	// ******************************
	//// end view stuff //
    }    

} // View3DS
